rm(list=ls())
setwd("/Users/kevinbakker/Desktop/VZV_data_code")
require(pomp);

param.sets1<- read.csv(file='VZV3_results2.csv',header=TRUE); #note these are earlier results of a pomp interation so we need to remove the log-likelihood, standard error, and seed.
param.sets2 <- param.sets1[,1:50]
param.sets <- cbind(param.sets2, 0,0)
colnames(param.sets)[51] <- 'mult'
colnames(param.sets)[52] <- 'expony'
modelnumber <- 2; # range for model 1-4
boosting <- 0;    # range for boost 0-1 (no/yes)
param.sets$model <- modelnumber;
param.sets$boost <- boosting;
output.name<- c("VZV_results");

#the following 4 lines are for running on a supercomputer. lines 19-20 are necessary even if running the model locally.
args <- (commandArgs(TRUE));  # command line arguments 
run <- as.numeric(args[1]);  # 
no.runs <- c(1); # number of cores
no.MC.reps <- c(1); #number of monte carlo reps you want to do. 

# load our pomp object
source('VZV_POsim_new.R')

#total searches
total.params<- dim(param.sets)[1]*no.MC.reps;
MIFed.params<-mat.or.vec(total.params,dim(param.sets)[2]+3);
MIFed.params<-as.data.frame(MIFed.params);
names(MIFed.params)<- c(names(param.sets),'LogLik','seLogLik','seed');
seeds <- ceiling(runif(n=total.params,min=1,max=2^30));
MIFed.params[,'seed'] <- seeds;

#Which parameters to estimate depending on model number. 
estnames <- c()
if(modelnumber ==1 && boosting == 0){
estnames <- c('zeta1','zeta2','zeta3','zeta4','zeta5','zeta6','rhovz','tauvz','tauhz','m1','m2','Beta_SD','S.0','IVZ.0','L1.0','IHZ.0','L2.0'); 
  }else if(modelnumber == 1 && boosting == 1){ 
estnames <- c('zeta1','zeta2','zeta3','zeta4','zeta5','zeta6','rhovz','tauvz','tauhz','m1','m2','Beta_SD','S.0','IVZ.0','L1.0','IHZ.0','L2.0','eta','omicron'); 
  }else if(modelnumber == 2 && boosting == 0){ 
estnames <- c('zeta1','zeta2','zeta3','zeta4','zeta5','zeta6','rhovz','tauvz','tauhz','m1','m2','Beta_SD','S.0','IVZ.0','L1.0','IHZ.0','L2.0','zeta1B','zeta2B','zeta3B','zeta4B','zeta5B','zeta6B'); 
  }else if(modelnumber == 2 && boosting == 1){ 
estnames <- c('zeta1','zeta2','zeta3','zeta4','zeta5','zeta6','rhovz','tauvz','tauhz','m1','m2','Beta_SD','S.0','IVZ.0','L1.0','IHZ.0','L2.0','zeta1B','zeta2B','zeta3B','zeta4B','zeta5B','zeta6B','eta','omicron'); 
  }else if(modelnumber == 3 && boosting == 0){
estnames <- c('zeta1','zeta2','zeta3','zeta4','zeta5','zeta6','rhovz','tauvz','tauhz','m1','m2','Beta_SD','S.0','IVZ.0','L1.0','IHZ.0','L2.0','a','nu','xi'); 
  }else if(modelnumber == 3 && boosting == 1){
estnames <- c('zeta1','zeta2','zeta3','zeta4','zeta5','zeta6','rhovz','tauvz','tauhz','m1','m2','Beta_SD','S.0','IVZ.0','L1.0','IHZ.0','L2.0','eta','omicron','a','nu','xi'); 
  }else if(modelnumber == 4 && boosting == 0){ 
estnames <- c('rhovz','tauvz','tauhz','m1','m2','Beta_SD','S.0','IVZ.0','L1.0','IHZ.0','L2.0','chi','upsilon','sigma','a','nu','xi'); 
  }else if(modelnumber == 4 && boosting == 1){
estnames <- c('rhovz','tauvz','tauhz','m1','m2','Beta_SD','S.0','IVZ.0','L1.0','IHZ.0','L2.0','eta','omicron','chi','upsilon','sigma','a','nu','xi');   

  }else{  print("You have an error")}
# Initial valued parameters we want to estimate 
estICs <- grep("0",estnames,value=TRUE);  #state variables to estimate (which we will do first) - grabs all parameters with a '0' in them
estpars <- setdiff(estnames,estICs);   # All other parameters we want to estimate

# Random-walk standard deviation. On the log and logit scale a value of 0.02 is about a 2% variation in the parameter value.
# You don't need to use the same rw.sds for each parameter. I have found that 2% works well when you have used a Sobol design.
#these are .1 for multiple iterations through mif (below)


valus <- rep(list(0.2),length(estnames))
names(valus) <- estnames[1:length(estnames)]
rwsd <- do.call(rw.sd,valus)

valus2 <- rep(list(0.2*0.5),length(estnames))
names(valus2) <- estnames[1:length(estnames)]
rwsd2 <- do.call(rw.sd,valus2)

valus3 <- rep(list(0.2*0.25),length(estnames))
names(valus3) <- estnames[1:length(estnames)]
rwsd3 <- do.call(rw.sd,valus3)

# set up run index, this is indexing the parameter set that we are using in this run
no.sets<- dim(param.sets)[1]
n<- floor(no.sets/no.runs)
start <- 1 + (run-1)*n # n is the number of param set in each run
end<- n + (run-1)*n 
if(run==no.runs){end=dim(param.sets)[1]}

#######################################################################################################
#MIF
write.seq<-rep(c(rep(0,9),1),length.out=dim(param.sets)[1]);   #how often the file writes

# for (i in start:end){ # for each parameter set   #this basically keeps them in chunks of which of the 50k+ param sets to run (i.e. 1-5000, 5001-1000, etc...)
for (i in 1:2){ # if you want to run the full sweep, use line 75, but line 76 runs 2 parameter sets and won't take days.
  current.params <- unlist(param.sets[i,1:length(param.sets[1,])]) # grab parameter set
  # for(j in 1:no.MC.reps){ # for each replicate  #how many times you want to repeat each parameter set
  for(j in 1:1){ #  same as above, if you run multiple reps, it will take days, but keeping this at 1 and i (line 76) at 2, will compile your C code and fit 2 parameter sets
    results.index<- ((i-1)*no.MC.reps) + j # calculate the index in our results data frame where we will put the results of this replicate
    try({
    save.rng.state <- .Random.seed 
    set.seed(seeds[results.index]) # set seed

    # Find some decent initial conditions for this parameter set
    mif.dat1 <- mif2(
                    my.model1,
                    Nmif=100,
                    start=current.params, # we will initialize using the parameter set resulting from "mif.ics" above.
                    pars=estnames, # list of parameters we are estimating 
                    ivps=estICs, # list of initial conditions we are estimating
                    rw.sd=rwsd,
                    Np=10000,
                    # var.factor=3,
                    cooling.type='hyperbolic',
                    cooling.fraction= 0.5,
                    # method='mif2',
                    max.fail=110
                   )  
    MIFed.pars1<- coef(mif.dat1) # MIFed parameter set

        #run MIF - I run it 3 times with increasingly smaller random walks (rw.sd within each loop). This takes longer initially, but speed up the overall process.
    mif.dat2 <- mif2(
                    my.model1,
                    Nmif=100,
                    start=MIFed.pars1, # we will initialize using the parameter set resulting from "mif.ics" above.
                    pars=estnames, # list of parameters we are estimating 
                    # ivps=estICs, # list of initial conditions we are estimating
                    rw.sd=rwsd,
                    Np=10000,
                    # var.factor=3,
                    cooling.type='hyperbolic',
                    cooling.fraction= 0.5,
                    # method='mif2',
                    max.fail=110
                   )  
    MIFed.pars2<- coef(mif.dat2) # MIFed parameter set

    mif.dat3 <- mif2(
                    my.model1,
                    Nmif=100,
                    start=MIFed.pars2, # we will initialize using the parameter set resulting from "mif.ics" above.
                    pars=estnames, # list of parameters we are estimating 
                    # ivps=estICs, # list of initial conditions we are estimating
                    rw.sd=rwsd2,
                    Np=10000,
                    # var.factor=3,
                    cooling.type='hyperbolic',
                    cooling.fraction= 0.5,
                    # method='mif2',
                    max.fail=110
                   )  
    MIFed.pars3<- coef(mif.dat3) # MIFed parameter set

    mif.dat4 <- mif2(
                    my.model1,
                    Nmif=100,
                    start=MIFed.pars3, # we will initialize using the parameter set resulting from "mif.ics" above.
                    pars=estnames, # list of parameters we are estimating 
                    # ivps=estICs, # list of initial conditions we are estimating
                    rw.sd=rwsd3,
                    Np=10000,
                    # var.factor=3,
                    cooling.type='hyperbolic',
                    cooling.fraction= 0.5,
                    # method='mif2',
                    max.fail=110
                   )  
    MIFed.pars4<- coef(mif.dat4) # MIFed parameter set
    .Random.seed <<- save.rng.state
         
    # calculate the likelihood of the MIFed parameter set
    loglik.mif<- replicate(n=10,logLik(pfilter(my.model1,params=MIFed.pars4,Np=5000,max.fail=110)))
      bl<- mean(loglik.mif)
      loglik.mif.est<- bl+log(mean(exp(loglik.mif-bl)))
      loglik.mif.se <- sd(exp(loglik.mif-bl))/sqrt(length(loglik.mif))/exp(loglik.mif.est-bl) 

    # put the results in our results data frame
    MIFed.params[results.index,1:length(param.sets[1,])]<-MIFed.pars4
    MIFed.params[results.index,'LogLik'] <-loglik.mif.est
    MIFed.params[results.index,'seLogLik'] <-loglik.mif.se
    MIFed.params <- as.data.frame(MIFed.params)
    })
  }
  # write to file every 10 parameter sets
  if(write.seq[i]==1){write.csv(MIFed.params,file=paste(c(output.name,'_run',run,'.csv'),collapse=''),row.names=FALSE)}
}


#######################################################################################################
# Write the final output
######################################################################################################
write.csv(MIFed.params,file=paste(c(output.name,'_run',run,'.csv'),collapse=''),row.names=FALSE)
